/*!
 * chai
 * Copyright(c) 2011 Jake Luer <jake@alogicalparadox.com>
 * MIT Licensed
 */

/**
 * ### TDD Style Introduction
 *
 * The TDD style is exposed through `assert` interfaces. This provides
 * the classic assert.`test` notation, similiar to that packaged with
 * node.js. This assert module, however, provides several additional
 * tests and is browser compatible.
 *
 *      // assert
 *      var assert = require('chai').assert;
 *        , foo = 'bar';
 *
 *      assert.typeOf(foo, 'string');
 *      assert.equal(foo, 'bar');
 */

/*!
 * Module dependencies.
 */
var Assertion = require('../assertion');

/*!
 * Module export.
 */

var assert = module.exports = {};

/**
 * # .ok(object, [message])
 *
 * Assert object is truthy.
 *
 *      assert.ok('everthing', 'everything is ok');
 *      assert.ok(false, 'this will fail');
 *
 * @name ok
 * @param {*} object to test
 * @param {String} message
 * @api public
 */

assert.ok = function (val, msg) {
  new Assertion(val, msg).is.ok;
};

/**
 * # .equal(actual, expected, [message])
 *
 * Assert strict equality.
 *
 *      assert.equal(3, 3, 'these numbers are equal');
 *
 * @name equal
 * @param {*} actual
 * @param {*} expected
 * @param {String} message
 * @api public
 */

assert.equal = function (act, exp, msg) {
  new Assertion(act, msg).to.equal(exp);
};

/**
 * # .notEqual(actual, expected, [message])
 *
 * Assert not strict equality.
 *
 *      assert.notEqual(3, 4, 'these numbers are not equal');
 *
 * @name notEqual
 * @param {*} actual
 * @param {*} expected
 * @param {String} message
 * @api public
 */

assert.notEqual = function (act, exp, msg) {
  new Assertion(act, msg).to.not.equal(exp);
};

/**
 * # .deepEqual(actual, expected, [message])
 *
 * Assert not deep equality.
 *
 *      assert.deepEqual({ tea: 'green' }, { tea: 'green' });
 *
 * @name deepEqual
 * @param {*} actual
 * @param {*} expected
 * @param {String} message
 * @api public
 */

assert.deepEqual = function (act, exp, msg) {
  new Assertion(act, msg).to.eql(exp);
};

/**
 * # .notDeepEqual(actual, expected, [message])
 *
 * Assert not deep equality.
 *
 *      assert.notDeepEqual({ tea: 'green' }, { tea: 'jasmine' });
 *
 * @name notDeepEqual
 * @param {*} actual
 * @param {*} expected
 * @param {String} message
 * @api public
 */

assert.notDeepEqual = function (act, exp, msg) {
  new Assertion(act, msg).to.not.eql(exp);
};

/**
 * # .isTrue(value, [message])
 *
 * Assert `value` is true.
 *
 *      var tea_served = true;
 *      assert.isTrue(tea_served, 'the tea has been served');
 *
 * @name isTrue
 * @param {Boolean} value
 * @param {String} message
 * @api public
 */

assert.isTrue = function (val, msg) {
  new Assertion(val, msg).is.true;
};

/**
 * # .isFalse(value, [message])
 *
 * Assert `value` is false.
 *
 *      var tea_served = false;
 *      assert.isFalse(tea_served, 'no tea yet? hmm...');
 *
 * @name isFalse
 * @param {Boolean} value
 * @param {String} message
 * @api public
 */

assert.isFalse = function (val, msg) {
  new Assertion(val, msg).is.false;
};

/**
 * # .isNull(value, [message])
 *
 * Assert `value` is null.
 *
 *      assert.isNull(err, 'no errors');
 *
 * @name isNull
 * @param {*} value
 * @param {String} message
 * @api public
 */

assert.isNull = function (val, msg) {
  new Assertion(val, msg).to.not.exist;
};

/**
 * # .isNotNull(value, [message])
 *
 * Assert `value` is not null.
 *
 *      var tea = 'tasty chai';
 *      assert.isNotNull(tea, 'great, time for tea!');
 *
 * @name isNotNull
 * @param {*} value
 * @param {String} message
 * @api public
 */

assert.isNotNull = function (val, msg) {
  new Assertion(val, msg).to.exist;
};

/**
 * # .isUndefined(value, [message])
 *
 * Assert `value` is undefined.
 *
 *      assert.isUndefined(tea, 'no tea defined');
 *
 * @name isUndefined
 * @param {*} value
 * @param {String} message
 * @api public
 */

assert.isUndefined = function (val, msg) {
  new Assertion(val, msg).to.equal(undefined);
};

/**
 * # .isFunction(value, [message])
 *
 * Assert `value` is a function.
 *
 *      var serve_tea = function () { return 'cup of tea'; };
 *      assert.isFunction(serve_tea, 'great, we can have tea now');
 *
 * @name isFunction
 * @param {Function} value
 * @param {String} message
 * @api public
 */

assert.isFunction = function (val, msg) {
  new Assertion(val, msg).to.be.a('function');
};

/**
 * # .isObject(value, [message])
 *
 * Assert `value` is an object.
 *
 *      var selection = { name: 'Chai', serve: 'with spices' };
 *      assert.isObject(selection, 'tea selection is an object');
 *
 * @name isObject
 * @param {Object} value
 * @param {String} message
 * @api public
 */

assert.isObject = function (val, msg) {
  new Assertion(val, msg).to.be.an('object');
};

/**
 * # .isArray(value, [message])
 *
 * Assert `value` is an instance of Array.
 *
 *      var menu = [ 'green', 'chai', 'oolong' ];
 *      assert.isArray(menu, 'what kind of tea do we want?');
 *
 * @name isArray
 * @param {*} value
 * @param {String} message
 * @api public
 */

assert.isArray = function (val, msg) {
  new Assertion(val, msg).to.be.instanceof(Array);
};

/**
 * # .isString(value, [message])
 *
 * Assert `value` is a string.
 *
 *      var teaorder = 'chai';
 *      assert.isString(tea_order, 'order placed');
 *
 * @name isString
 * @param {String} value
 * @param {String} message
 * @api public
 */

assert.isString = function (val, msg) {
  new Assertion(val, msg).to.be.a('string');
};

/**
 * # .isNumber(value, [message])
 *
 * Assert `value` is a number
 *
 *      var cups = 2;
 *      assert.isNumber(cups, 'how many cups');
 *
 * @name isNumber
 * @param {Number} value
 * @param {String} message
 * @api public
 */

assert.isNumber = function (val, msg) {
  new Assertion(val, msg).to.be.instanceof(Number);
};

/**
 * # .isBoolean(value, [message])
 *
 * Assert `value` is a boolean
 *
 *      var teaready = true
 *        , teaserved = false;
 *
 *      assert.isBoolean(tea_ready, 'is the tea ready');
 *      assert.isBoolean(tea_served, 'has tea been served');
 *
 * @name isBoolean
 * @param {*} value
 * @param {String} message
 * @api public
 */

assert.isBoolean = function (val, msg) {
  new Assertion(val, msg).to.be.a('boolean');
};

/**
 * # .typeOf(value, name, [message])
 *
 * Assert typeof `value` is `name`.
 *
 *      assert.typeOf('tea', 'string', 'we have a string');
 *
 * @name typeOf
 * @param {*} value
 * @param {String} typeof name
 * @param {String} message
 * @api public
 */

assert.typeOf = function (val, type, msg) {
  new Assertion(val, msg).to.be.a(type);
};

/**
 * # .instanceOf(object, constructor, [message])
 *
 * Assert `value` is instanceof `constructor`.
 *
 *      var Tea = function (name) { this.name = name; }
 *        , Chai = new Tea('chai');
 *
 *      assert.instanceOf(Chai, Tea, 'chai is an instance of tea');
 *
 * @name instanceOf
 * @param {Object} object
 * @param {Constructor} constructor
 * @param {String} message
 * @api public
 */

assert.instanceOf = function (val, type, msg) {
  new Assertion(val, msg).to.be.instanceof(type);
};

/**
 * # .include(value, includes, [message])
 *
 * Assert the inclusion of an object in another.
 *
 *      var obj = {foo: 'bar', baz: {baaz: 42}, qux: 13};
 *      assert.include(obj, {foo: 'bar'}, 'object contains subobject');
 *
 *      assert.include('foobar', 'bar', 'foobar contains string `var`);
 *      assert.include([ 1, 2, 3], 3, 'array contains value);
 *
 * @name include
 * @param {Array|String|Object} value
 * @param {*} includes
 * @param {String} message
 * @api public
 */

assert.include = function (exp, inc, msg) {
  var obj = new Assertion(exp, msg);

  if (Array.isArray(exp)) {
    obj.to.contain(inc);
  } else if ('string' === typeof exp) {
    obj.to.include.string(inc);
  } else {
    obj.to.include.object(inc);
  }
};

/**
 * # .match(value, constructor, [message])
 *
 * Assert that `value` matches regular expression.
 *
 *      assert.match('foobar', /^foo/, 'Regexp matches');
 *
 * @name match
 * @param {*} value
 * @param {RegExp} RegularExpression
 * @param {String} message
 * @api public
 */

assert.match = function (exp, re, msg) {
  new Assertion(exp, msg).to.match(re);
};

/**
 * # .length(value, constructor, [message])
 *
 * Assert that object has expected length.
 *
 *      assert.length([1,2,3], 3, 'Array has length of 3');
 *      assert.length('foobar', 5, 'String has length of 6');
 *
 * @name length
 * @param {*} value
 * @param {Number} length
 * @param {String} message
 * @api public
 */

assert.length = function (exp, len, msg) {
  new Assertion(exp, msg).to.have.length(len);
};

/**
 * # .throw(function, constructor, [message])
 *
 * Assert that a function will throw a specific
 * type of error.
 *
 *      var fn = function () { throw new ReferenceError(''); }
 *      assert.throw(fn, ReferenceError, 'function throw reference error');
 *
 * @name throw
 * @alias throws
 * @param {Function} function to test
 * @param {ErrorConstructor} constructor
 * @param {String} message
 * @see https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Error#Error_types
 * @api public
 */

assert.throw = function (fn, type, msg) {
  new Assertions(fn, msg).to.throw(type);
};

/*!
 * Aliases.
 */

(function alias(name, as){
  assert[as] = assert[name];
  return alias;
})
('length', 'lengthOf');
//('keys', 'key')
//('ownProperty', 'haveOwnProperty')
//('above', 'greaterThan')
//('below', 'lessThan')
//('throw', 'throws');